#
from enigma import *
from Screens.Standby import TryQuitMainloop, Standby
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox
from Screens.ChoiceBox import ChoiceBox
from Components.ActionMap import ActionMap, NumberActionMap
from Components.ScrollLabel import ScrollLabel
from Components.GUIComponent import *
from Components.MenuList import MenuList
from Components.Input import Input
from Screens.Console import Console
from Plugins.Plugin import PluginDescriptor
from Screens.ImageWizard import ImageWizard
from Screens.ServiceInfo import *
from Plugins.Plugin import PluginDescriptor
from Tools import Notifications
from Components.config import config

import os
import keymapparser
session = None

from time import *
import time
import datetime

pauli_readme = "/usr/lib/enigma2/python/Plugins/Extensions/Pauli/readme.txt"
pauli_neutrino_keymap = "/usr/lib/enigma2/python/Plugins/Extensions/Pauli/keymap_neutrino.xml"
pauli_keymap = "/usr/share/enigma2/keymap.xml"
pauli_mytest = "/usr/lib/enigma2/python/mytest.py"
pauli_recordinfo = "/usr/lib/enigma2/python/RecordTimer.py"
pauli_timerentry = "/usr/lib/enigma2/python/Screens/TimerEntry.py"
pauli_channelonboot = "/etc/enigma2/channelonboot"
standbyonbootfile = "/etc/enigma2/paulistandbyonboot"

pauli_pluginversion = "4.7"


def autostart(reason, **kwargs):
    global session
    if os.path.exists(standbyonbootfile) == True:
       if reason == 0 and kwargs.has_key("session"):
          session = kwargs["session"]
          session.open(PauliStartup)
       
def main(session, **kwargs):
    session.open(Pauli)

def Plugins(**kwargs):
    return [PluginDescriptor(where = [PluginDescriptor.WHERE_SESSIONSTART, PluginDescriptor.WHERE_AUTOSTART], fnc = autostart),PluginDescriptor(name="Pauli Neutrino Keymap", description="Switch Enigma2 and Neutrino Keymap", where = PluginDescriptor.WHERE_PLUGINMENU, icon="pauli.png", fnc=main)]

class Pauli(Screen):
    skin = """
        <screen position="200,200" size="370,300" title="Pauli Neutrino Keymap" >
            <widget name="menu" position="10,10" size="360,290" scrollbarMode="showOnDemand" />
        </screen>"""
        
    def __init__(self, session, args = 0):
        self.skin = Pauli.skin
        self.session = session
        Screen.__init__(self, session)
        self.menu = args
        list = []

        list.append((_("Change Keymap"), "togglekeymap"))        
        list.append((_("Change Right/Left"), "togglelistbox"))        
        list.append((_("Change Standby on boot"), "standbyonboot"))        
        list.append((_("Change Deepstandby"), "deepstandby"))        
        list.append((_("Change Recording Timer Info"), "recording"))        
        list.append((_("Change Default Timer Action "), "timeraction"))        
        list.append((_("Change Starting Channel"), "channelonboot"))        
        list.append((_("Change Systemtime"), "changetime"))        
        list.append((_("Restart Enigma2"), "restarte2"))        
        list.append((_("readme.txt"), "readme"))        
        list.append((_("about"), "about"))        
        self["menu"] = MenuList(list)
        self["actions"] = ActionMap(["WizardActions", "DirectionActions"],{"ok": self.go,"back": self.close,}, -1)
        
    def go(self):
        returnValue = self["menu"].l.getCurrentSelection()[1]
        if returnValue is not None:
           if returnValue is "togglekeymap":
              ToggleKeymap(self.session)
           if returnValue is "togglelistbox":
              ToggleListbox(self.session)
           elif returnValue is "deepstandby":
              ToggleDeepstandby(self.session)
           elif returnValue is "standbyonboot":
              if os.path.exists(standbyonbootfile) == True:
                 os.system("rm %s" % standbyonbootfile)
	         self.session.open(MessageBox,"Standby on boot disabled !", MessageBox.TYPE_INFO)
              else:
                 os.system("touch %s" % standbyonbootfile)
	         self.session.open(MessageBox,"Standby on boot enabled !", MessageBox.TYPE_INFO)
           elif returnValue is "recording":
              ToggleRecordTimer(self.session)
           elif returnValue is "timeraction":
              ToggleTimerAction(self.session)
           elif returnValue is "channelonboot":
              ChannelonBoot(self.session)
           elif returnValue is "changetime":
              ChangeSystemTime(self.session)
           elif returnValue is "restarte2":
              TryQuitMainloop(self.session,3)
           elif returnValue is "readme":
              self.session.open(Console,_("readme.txt of Pauli Neutrino Keymap"),["cat %s" % pauli_readme])
           elif returnValue is "about":
              self.session.open(MessageBox,_("Pauli Neutrino Keymap Plugin Version %s\nby romeo-golf, powerd by gutemine\n\nWolfgang Pauli was an Austrian physicist and\nNobel price winner and postulated the Neutrino particle" % pauli_pluginversion), MessageBox.TYPE_INFO)
              
class ToggleKeymap(Screen):
    def __init__(self, session):

        self.session = session
        keymapfile = open(pauli_keymap, "r")
        text=keymapfile.read()
        keymapfile.close()
        if text.find("_Neutrino") is -1:

           # now we are in trouble, because Neutrino extensions are gone 
           # probably enigma2 upgrade brought back virgin keymap :-)          
           # so lets bring it back to normal and don't cry :-)

           print "re-patching keymap"
           keymapneutrinofile = open(pauli_neutrino_keymap, "r")
           neutrinotext=keymapneutrinofile.read()
           keymapneutrinofile.close()
           text=text.replace("</keymap>","")
           text=text+neutrinotext+"</keymap>"
           keymapfile = open(pauli_keymap, "w")
           keymapfile.write(text)
           keymapfile.close()
            
        keymapfile = open(pauli_keymap, "r")
        text2=keymapfile.readline()
#       second line shows if Neutrino is active
        text2=keymapfile.readline()
        keymapfile.close()

	text=text.replace("ListboxActions","ListboxActions_Neutrino")
	text=text.replace("InfobarChannelSelection","InfobarChannelSelection_Neutrino")
	text=text.replace("InfobarShowHideActions","InfobarShowHideActions_Neutrino")
	text=text.replace("InfobarSubserviceSelectionActions","InfobarSubserviceSelectionActions_Neutrino")
	text=text.replace("InfobarSubserviceQuickzapActions","InfobarSubserviceQuickzapActions_Neutrino")
	text=text.replace("ChannelSelectBaseActions","ChannelSelectBaseActions_Neutrino")
	text=text.replace("_Neutrino_Neutrino","")

        keymapfile = open(pauli_keymap, "w")
        keymapfile.write(text)
        keymapfile.close()
        keymapparser.removeKeymap(pauli_keymap)
        keymapparser.readKeymap(pauli_keymap)
        if text2.find("_Neutrino") is -1:
           self.session.open(MessageBox,_("Pauli switched Enigma2 to Neutrino Keymap !"), MessageBox.TYPE_INFO)
        else:
           self.session.open(MessageBox,_("Pauli switched Neutrino to Enigma2 Keymap !"), MessageBox.TYPE_INFO)

        
class ToggleListbox(Screen):
    def __init__(self, session):

        self.session = session
        keymapfile = open(pauli_keymap, "r")
        text=keymapfile.read()
        keymapfile.close()
        if text.find("_Neutrino") is -1:

           # now we are in trouble, because Neutrino extensions are gone 
           # probably enigma2 upgrade brought back virgin keymap :-)          
           # so lets bring it back to normal and don't cry :-)

           print "re-patching keymap"
           keymapneutrinofile = open(pauli_neutrino_keymap, "r")
           neutrinotext=keymapneutrinofile.read()
           keymapneutrinofile.close()
           text=text.replace("</keymap>","")
           text=text+neutrinotext+"</keymap>"
           keymapfile = open(pauli_keymap, "w")
           keymapfile.write(text)
           keymapfile.close()
            
	if text.find("<!-- key id=\"KEY_LEFT\" mapto=\"pageUp\" flags=\"mr\" />") is -1:
	   text=text.replace("<key id=\"KEY_LEFT\" mapto=\"pageUp\" flags=\"mr\" />","<!-- key id=\"KEY_LEFT\" mapto=\"pageUp\" flags=\"mr\" />")
	   text=text.replace("<key id=\"KEY_RIGHT\" mapto=\"pageDown\" flags=\"mr\" />","<key id=\"KEY_RIGHT\" mapto=\"pageDown\" flags=\"mr\" /-->")
	else:
	   text=text.replace("<!-- key id=\"KEY_LEFT\" mapto=\"pageUp\" flags=\"mr\" />","<key id=\"KEY_LEFT\" mapto=\"pageUp\" flags=\"mr\" />")
	   text=text.replace("<key id=\"KEY_RIGHT\" mapto=\"pageDown\" flags=\"mr\" /-->","<key id=\"KEY_RIGHT\" mapto=\"pageDown\" flags=\"mr\" />")

        keymapfile = open(pauli_keymap, "w")
        keymapfile.write(text)
        keymapfile.close()
        keymapparser.removeKeymap(pauli_keymap)
        keymapparser.readKeymap(pauli_keymap)
	if text.find("<!-- key id=\"KEY_LEFT\" mapto=\"pageUp\" flags=\"mr\" />") is -1:
           self.session.open(MessageBox,_("Pauli enabled Right/Left in Listbox !"), MessageBox.TYPE_INFO)
        else:
           self.session.open(MessageBox,_("Pauli disabled Right/Left in Listbox !"), MessageBox.TYPE_INFO)

        
class ToggleDeepstandby(Screen):
    def __init__(self, session):

        self.session = session
        keymapfile = open(pauli_keymap, "r")
        text=keymapfile.read()
        keymapfile.close()
        if text.find("power_long") is -1:
           # old image - so let's patch mytest.py
           mytestfile = open(pauli_mytest, "r")
           text=mytestfile.read()
           mytestfile.close()
           if text.find("self.powerKeyTimer.start(3000, True)") is -1:
	      text=text.replace("self.powerKeyTimer.start(0000, True)","self.powerKeyTimer.start(3000, True)")
              mytestfile = open(pauli_mytest, "w")
              mytestfile.write(text)
              mytestfile.close()
              self.session.openWithCallback(self.DoDeepstandbyRestart,MessageBox,_("Enabled Deepstandby after 3 sec, restart enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)
           else:
	      text=text.replace("self.powerKeyTimer.start(3000, True)","self.powerKeyTimer.start(0000, True)")
              mytestfile = open(pauli_mytest, "w")
              mytestfile.write(text)
              mytestfile.close()
              self.session.openWithCallback(self.DoDeepstandbyRestart,MessageBox,_("Enabled Deepstandby immediate, restart enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)
        else:
           # new image - so let's patch keymap.xml
           if text.find("<key id=\"KEY_POWER\" mapto=\"power_long\" flags=\"l\" />") is -1:
              text=text.replace("<key id=\"KEY_POWER\" mapto=\"power_long\" flags=\"m\" />","<key id=\"KEY_POWER\" mapto=\"power_long\" flags=\"l\" />")
	      text=text.replace("<key id=\"KEY_POWER\" mapto=\"power_down\" flags=\"l\" />","<key id=\"KEY_POWER\" mapto=\"power_down\" flags=\"m\" />")
              keymapfile = open(pauli_keymap, "w")
              keymapfile.write(text)
              keymapfile.close()
              self.session.openWithCallback(self.DoDeepstandbyRestart,MessageBox,_("Enabled Deepstandby with power short, restart enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)
           else:
	      text=text.replace("<key id=\"KEY_POWER\" mapto=\"power_long\" flags=\"l\" />","<key id=\"KEY_POWER\" mapto=\"power_long\" flags=\"m\" />")
	      text=text.replace("<key id=\"KEY_POWER\" mapto=\"power_down\" flags=\"m\" />","<key id=\"KEY_POWER\" mapto=\"power_down\" flags=\"l\" />")
              keymapfile = open(pauli_keymap, "w")
              keymapfile.write(text)
              keymapfile.close()
              self.session.openWithCallback(self.DoDeepstandbyRestart,MessageBox,_("Enabled Deepstandby with power long, restart enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)
            
    def DoDeepstandbyRestart(self,answer):
        if answer is None:
            self.skipDeepstandbyToggle("answer is None, restart later")
        if answer is False:
            self.skipDeepstandbyToggle("you were not confirming, restart later")
        else:
            TryQuitMainloop(self.session,3)
            
    def skipDeepstandbyToggle(self,reason):
        self.session.open(MessageBox,_("Enigma2 restart was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
class ToggleRecordTimer(Screen):
    def __init__(self, session):

        self.session = session
        recordfile = open(pauli_recordinfo, "r")
        text=recordfile.read()
        recordfile.close()
        if text.find("pass # Notifications.AddPopup(text = _(\"A record has been started:") is -1:
	   text=text.replace("Notifications.AddPopup(text = _(\"A record has been started:","pass # Notifications.AddPopup(text = _(\"A record has been started:")
           recordfile = open(pauli_recordinfo, "w")
           recordfile.write(text)
           recordfile.close()
           self.session.openWithCallback(self.DoRecordTimerRestart,MessageBox,_("disabeld Record Timer Message, restarting enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)
        else:
	   text=text.replace("pass # Notifications.AddPopup(text = _(\"A record has been started:","Notifications.AddPopup(text = _(\"A record has been started:")
           recordfile = open(pauli_recordinfo, "w")
           recordfile.write(text)
           recordfile.close()
           self.session.openWithCallback(self.DoRecordTimerRestart,MessageBox,_("enabeld Record Timer Message, restarting enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)

    def DoRecordTimerRestart(self,answer):
        if answer is None:
            self.skipRecordTimerToggle("answer is None, restart later")
        if answer is False:
            self.skipRecordTimerToggle("you were not confirming, restart later")
        else:
            TryQuitMainloop(self.session,3)
            
    def skipRecordTimerToggle(self,reason):
        self.session.open(MessageBox,_("Enigma2 restart was canceled, because %s" % reason), MessageBox.TYPE_WARNING)

class ToggleTimerAction(Screen):
    def __init__(self, session):

        self.session = session
        actionfile = open(pauli_timerentry, "r")
        text=actionfile.read()
        actionfile.close()
        if text.find("default = afterevent") is -1:
	   text=text.replace("default = \"deepstandby\"","default = afterevent")
           actionfile = open(pauli_timerentry, "w")
           actionfile.write(text)
           actionfile.close()
           self.session.openWithCallback(self.DoTimerActionRestart,MessageBox,_("disabeld Default Timer Action deepstandby, restarting enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)
        else:
	   text=text.replace("default = afterevent","default = \"deepstandby\"")
           actionfile = open(pauli_timerentry, "w")
           actionfile.write(text)
           actionfile.close()
           self.session.openWithCallback(self.DoTimerActionRestart,MessageBox,_("enabeld Default Timer Action deepstandby, restarting enigma2 to activate - OK ?"), MessageBox.TYPE_YESNO)

    def DoTimerActionRestart(self,answer):
        if answer is None:
            self.skipTimerActionToggle("answer is None, restart later")
        if answer is False:
            self.skipTimerActionToggle("you were not confirming, restart later")
        else:
            TryQuitMainloop(self.session,3)
            
    def skipTimerActionToggle(self,reason):
        self.session.open(MessageBox,_("Enigma2 restart was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
        
class ChannelonBoot(Screen):
    def __init__(self, session):

        self.session = session
        self.askForChannelonBoot()

    def askForChannelonBoot(self):
        self.session.openWithCallback(self.DoChannelonBoot,MessageBox,_("Use current Channel as Startup Channel ?"), MessageBox.TYPE_YESNO)

    def DoChannelonBoot(self,answer):
        if answer is None:
            self.skipChannelonBoot("answer is None")
        if answer is False:
            self.skipChannelonBoot("you were not confirming")
        else:
            paulifile = open(pauli_channelonboot, "w")
            paulifile.write("config.tv.lastroot=%s\n" % config.tv.lastroot.value)
            paulifile.write("config.tv.lastservice=%s\n" % config.tv.lastservice.value)
            paulifile.close()
            self.session.open(MessageBox,_("Current Channel as Startservice is done !"), MessageBox.TYPE_INFO)
            
    def skipChannelonBoot(self,reason):
        self.session.open(MessageBox,_("Current Channel as Startup Channel was canceled, because %s" % reason), MessageBox.TYPE_WARNING)

class ChangeSystemTime(Screen):
    def __init__(self, session):

        self.session = session

        jetzt = time.time()
        timezone = datetime.datetime.utcnow()
        delta = (jetzt - time.mktime(timezone.timetuple())) 
        print "delta: %i" % delta
        print "oldtime: %i" % jetzt
        # always add 1 min so that there is time for typing
        self.oldtime = strftime("%Y:%m:%d %H:%M",localtime())
        self.session.openWithCallback(self.askForNewTime,InputBox, title=_("Enter new Systemtime"), text="%s" % (self.oldtime), maxSize=16, type=Input.NUMBER)

    def askForNewTime(self,newclock):
        try:
           length=len(newclock)
        except:
           length=0
        if newclock is None:
            self.skipChangeTime("no new time")
        elif (length == 16) is False:
           self.skipChangeTime("new time string too short")
        elif (newclock.count(" ") < 1) is True:
            self.skipChangeTime("invalid format")
        elif (newclock.count(":") < 3) is True:
            self.skipChangeTime("invalid format")
        else:
            full=[]
            full=newclock.split(" ",1)
            newdate=full[0]
            newtime=full[1]
            print "newdate %s newtime %s" % (newdate, newtime)
            parts=[]
            parts=newdate.split(":",2)
            newyear=parts[0]
            newmonth=parts[1]
            newday=parts[2]
            parts=newtime.split(":",1)
            newhour=parts[0]
            newmin=parts[1]
            #
            # da some checks to make sure that date & time are OK !
            #
            maxmonth = 31
            if (int(newmonth) == 4) or (int(newmonth) == 6) or (int(newmonth) == 9) or (int(newmonth) == 11) is True:
               maxmonth=30
            elif (int(newmonth) == 2) is True:
               if ((4*int(int(newyear)/4)) == int(newyear)) is True:
                  maxmonth=28
               else:
                  maxmonth=27
            if (int(newyear) < 2007) or (int(newyear) > 2027)  or (len(newyear) < 4) is True:
               self.skipChangeTime("invalid year %s" %newyear)
            elif (int(newmonth) < 0) or (int(newmonth) >12) or (len(newmonth) < 2) is True:
               self.skipChangeTime("invalid month %s" %newmonth)
            elif (int(newday) < 1) or (int(newday) > maxmonth) or (len(newday) < 2) is True:              self.skipChangeTime("invalid day %s" %newday)
            elif (int(newhour) < 0) or (int(newhour) > 23) or (len(newhour) < 2) is True:
               self.skipChangeTime("invalid hour %s" %newhour)
            elif (int(newmin) < 0) or (int(newmin) > 59) or (len(newmin) < 2) is True:
               self.skipChangeTime("invalid minute %s" %newmin)
            else:
#              self.newtime = mktime(strptime(newtime,"%Y:%m:%d %H:%M"))
               self.newtime = "%s%s%s%s%s" %(newmonth,newday,newhour,newmin,newyear)
               print "date %s" % self.newtime
               self.session.openWithCallback(self.DoChangeTimeRestart,MessageBox,_("Enigma2 needs to be restarted to activate new system time - OK ?"), MessageBox.TYPE_YESNO)

    def DoChangeTimeRestart(self,answer):
        if answer is None:
            self.skipChangeTime("answer is None")
        if answer is False:
            self.skipChangeTime("you were not confirming")
        else:
            os.system("date %s" % (self.newtime))
            quitMainloop(3)

    def skipChangeTime(self,reason):
        self.session.open(MessageBox,_("System Time was NOT changed, because %s" % reason), MessageBox.TYPE_WARNING)


class PauliStartup(Screen):
    skin = """
        <screen position="150,100" size="400,300" title="Pauli Startup" >
        </screen>"""
    def __init__(self,session):
        self.skin = PauliStartup.skin
	self.session = session
	Screen.__init__(self,session)
   	self.TimerPauliStandby = eTimer()
	self.TimerPauliStandby.stop()
	self.TimerPauliStandby.timeout.get().append(self.CheckStandbyPauli)
	self.TimerPauliStandby.start(6000,True) 

    def CheckStandbyPauli(self):
	self.TimerPauliStandby.stop()
	self.session.openWithCallback(self.DoPauliStandby,MessageBox,"Standby after Boot enabled - OK ?",type = MessageBox.TYPE_YESNO,timeout = 20)		


    def DoPauliStandby(self,retval):
	if (retval):
           os.system("wall 'Pauli does Standby now'")
           Notifications.AddNotification(Standby)


